import * as tseslint from '@typescript-eslint/utils/ts-eslint';
import { ReportDescriptor } from '@typescript-eslint/utils/ts-eslint';
import { CompilerOptions, JsxEmit } from 'typescript';
import * as valibot from 'valibot';
import { InferOutput } from 'valibot';
import { _ } from '@eslint-react/eff';
import { TSESTree } from '@typescript-eslint/utils';

/**
 * Rule severity.
 * @since 0.0.1
 */
type RuleSeverity = "error" | "off" | "warn";
/**
 * Rule declaration.
 * @internal
 * @since 0.0.1
 */
type RuleDeclaration = [RuleSeverity, Record<string, unknown>?] | RuleSeverity;
/**
 * Rule config preset.
 * @since 0.0.1
 */
type RulePreset = Record<string, RuleDeclaration>;
/**
 * Rule context.
 * @since 0.0.1
 */
type RuleContext<MessageIds extends string = string, Options extends readonly unknown[] = readonly unknown[]> = tseslint.RuleContext<MessageIds, Options>;
/**
 * Rule feature.
 * @since 1.20.0
 */
type RuleFeature = "CFG" | "DBG" | "FIX" | "MOD" | "TSC" | "EXP";

type JsxConfig = Pick<CompilerOptions, "reactNamespace" | "jsx" | "jsxFactory" | "jsxFragmentFactory" | "jsxImportSource">;
/**
 * Create a JsxConfig object
 * @returns JsxConfig
 */
declare function make$2(): JsxConfig;
/**
 * Get JsxConfig from RuleContext
 * @param context The RuleContext
 * @returns JsxConfig
 */
declare function getFromContext$1(context: RuleContext): {
    jsx: JsxEmit;
    jsxFactory: string;
    jsxFragmentFactory: string;
    jsxImportSource: string;
    reactNamespace: string;
};
/**
 * Get JsxConfig from annotation
 * @param context The RuleContext
 * @returns JsxConfig
 */
declare function getFromAnnotation(context: RuleContext): JsxConfig;

type index$1_JsxConfig = JsxConfig;
declare const index$1_getFromAnnotation: typeof getFromAnnotation;
declare namespace index$1 {
  export { type index$1_JsxConfig as JsxConfig, index$1_getFromAnnotation as getFromAnnotation, getFromContext$1 as getFromContext, make$2 as make };
}

/**
 * @internal
 */
declare const LanguagePreferenceSchema: valibot.ObjectSchema<{
    readonly indentStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"tab", undefined>, valibot.LiteralSchema<"space", undefined>], undefined>, "space">;
    readonly indentWidth: valibot.OptionalSchema<valibot.NumberSchema<undefined>, 2>;
    readonly quoteStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"single", undefined>, valibot.LiteralSchema<"double", undefined>], undefined>, "single">;
    readonly semicolons: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"always", undefined>, valibot.LiteralSchema<"asNeeded", undefined>], undefined>, "always">;
    readonly trailingCommas: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"all", undefined>, valibot.LiteralSchema<"es5", undefined>, valibot.LiteralSchema<"none", undefined>], undefined>, "all">;
    readonly jsxQuoteStyle: valibot.OptionalSchema<valibot.UnionSchema<[valibot.LiteralSchema<"single", undefined>, valibot.LiteralSchema<"double", undefined>], undefined>, "double">;
}, undefined>;

/**
 * @internal
 */
type LanguagePreference = InferOutput<typeof LanguagePreferenceSchema>;
/**
 * The default language preference.
 */
declare const DEFAULT_LANGUAGE_PREFERENCE: {
    readonly indentStyle: "space";
    readonly indentWidth: 2;
    readonly jsxQuoteStyle: "double";
    readonly quoteStyle: "single";
    readonly semicolons: "always";
    readonly trailingCommas: "all";
};
/**
 * Get a copy of the default LanguagePreference.
 */
declare function make$1(): LanguagePreference;
declare function getFromContext(): void;
declare module "@typescript-eslint/utils/ts-eslint" {
    interface SharedConfigurationSettings {
    }
}

declare const index_DEFAULT_LANGUAGE_PREFERENCE: typeof DEFAULT_LANGUAGE_PREFERENCE;
type index_LanguagePreference = LanguagePreference;
declare const index_LanguagePreferenceSchema: typeof LanguagePreferenceSchema;
declare const index_getFromContext: typeof getFromContext;
declare namespace index {
  export { index_DEFAULT_LANGUAGE_PREFERENCE as DEFAULT_LANGUAGE_PREFERENCE, type index_LanguagePreference as LanguagePreference, index_LanguagePreferenceSchema as LanguagePreferenceSchema, index_getFromContext as getFromContext, make$1 as make };
}

/**
 * Regular expression for matching a TypeScript file extension.
 */
declare const TS_EXT: RegExp;
/**
 * Regular expression for matching a JavaScript file extension.
 */
declare const JS_EXT: RegExp;
/**
 * Regular expression for matching a PascalCase string.
 */
declare const PASCAL_CASE: RegExp;
/**
 * Regular expression for matching a camelCase string.
 */
declare const CAMEL_CASE: RegExp;
/**
 * Regular expression for matching a kebab-case string.
 */
declare const KEBAB_CASE: RegExp;
/**
 * Regular expression for matching a snake_case string.
 */
declare const SNAKE_CASE: RegExp;
/**
 * Regular expression for matching a CONSTANT_CASE string.
 */
declare const CONSTANT_CASE: RegExp;
declare const JAVASCRIPT_PROTOCOL: RegExp;
/**
 * Regular expression for matching a valid JavaScript identifier.
 */
declare const JS_IDENTIFIER: RegExp;
/**
 * Regular expression for matching a RegExp string.
 */
declare const REGEXP_STR: RegExp;
/**
 * Regular expression for matching a `@jsx` annotation comment.
 */
declare const ANNOTATION_JSX: RegExp;
/**
 * Regular expression for matching a `@jsxFrag` annotation comment.
 */
declare const ANNOTATION_JSX_FRAG: RegExp;
/**
 * Regular expression for matching a `@jsxRuntime` annotation comment.
 */
declare const ANNOTATION_JSX_RUNTIME: RegExp;
/**
 * Regular expression for matching a `@jsxImportSource` annotation comment.
 */
declare const ANNOTATION_JSX_IMPORT_SOURCE: RegExp;
/**
 * Regular expression for matching a React component name.
 */
declare const COMPONENT_NAME: RegExp;
/**
 * Regular expression for matching a React component name (loose).
 */
declare const COMPONENT_NAME_LOOSE: RegExp;
/**
 * Regular expression for matching a React Hook name.
 */
declare const HOOK_NAME: RegExp;
/**
 * Convert a string to the `RegExp`.
 * Normal strings (e.g. `"foo"`) is converted to `/^foo$/` of `RegExp`.
 * Strings like `"/^foo/i"` are converted to `/^foo/i` of `RegExp`.
 * @see https://github.com/sveltejs/eslint-plugin-svelte/blob/main/packages/eslint-plugin-svelte/src/utils/regexp.ts
 * @param string The string to convert.
 * @returns Returns the `RegExp`.
 */
declare function toRegExp(string: string): {
    test(s: string): boolean;
};
/**
 * Checks whether given string is regexp string
 * @param string The string to check
 * @returns boolean
 */
declare function isRegExp(string: string): boolean;

declare const RE_ANNOTATION_JSX: typeof ANNOTATION_JSX;
declare const RE_ANNOTATION_JSX_FRAG: typeof ANNOTATION_JSX_FRAG;
declare const RE_ANNOTATION_JSX_IMPORT_SOURCE: typeof ANNOTATION_JSX_IMPORT_SOURCE;
declare const RE_ANNOTATION_JSX_RUNTIME: typeof ANNOTATION_JSX_RUNTIME;
declare const RE_CAMEL_CASE: typeof CAMEL_CASE;
declare const RE_COMPONENT_NAME: typeof COMPONENT_NAME;
declare const RE_COMPONENT_NAME_LOOSE: typeof COMPONENT_NAME_LOOSE;
declare const RE_CONSTANT_CASE: typeof CONSTANT_CASE;
declare const RE_HOOK_NAME: typeof HOOK_NAME;
declare const RE_JAVASCRIPT_PROTOCOL: typeof JAVASCRIPT_PROTOCOL;
declare const RE_JS_EXT: typeof JS_EXT;
declare const RE_JS_IDENTIFIER: typeof JS_IDENTIFIER;
declare const RE_KEBAB_CASE: typeof KEBAB_CASE;
declare const RE_PASCAL_CASE: typeof PASCAL_CASE;
declare const RE_REGEXP_STR: typeof REGEXP_STR;
declare const RE_SNAKE_CASE: typeof SNAKE_CASE;
declare const RE_TS_EXT: typeof TS_EXT;
declare const RE_isRegExp: typeof isRegExp;
declare const RE_toRegExp: typeof toRegExp;
declare namespace RE {
  export { RE_ANNOTATION_JSX as ANNOTATION_JSX, RE_ANNOTATION_JSX_FRAG as ANNOTATION_JSX_FRAG, RE_ANNOTATION_JSX_IMPORT_SOURCE as ANNOTATION_JSX_IMPORT_SOURCE, RE_ANNOTATION_JSX_RUNTIME as ANNOTATION_JSX_RUNTIME, RE_CAMEL_CASE as CAMEL_CASE, RE_COMPONENT_NAME as COMPONENT_NAME, RE_COMPONENT_NAME_LOOSE as COMPONENT_NAME_LOOSE, RE_CONSTANT_CASE as CONSTANT_CASE, RE_HOOK_NAME as HOOK_NAME, RE_JAVASCRIPT_PROTOCOL as JAVASCRIPT_PROTOCOL, RE_JS_EXT as JS_EXT, RE_JS_IDENTIFIER as JS_IDENTIFIER, RE_KEBAB_CASE as KEBAB_CASE, RE_PASCAL_CASE as PASCAL_CASE, RE_REGEXP_STR as REGEXP_STR, RE_SNAKE_CASE as SNAKE_CASE, RE_TS_EXT as TS_EXT, RE_isRegExp as isRegExp, RE_toRegExp as toRegExp };
}

declare function make<TMessageID extends string>(context: RuleContext<TMessageID>): {
    readonly send: (descriptor: _ | null | ReportDescriptor<TMessageID>) => void;
    readonly sendOrElse: <TElse>(descriptor: _ | null | ReportDescriptor<TMessageID>, fallback: () => TElse) => void | TElse;
};

declare const Report_make: typeof make;
declare namespace Report {
  export { Report_make as make };
}

type ImplicitReturnArrowFunctionExpression = TSESTree.ArrowFunctionExpression & {
    body: TSESTree.Expression;
};
type ObjectDestructuringVariableDeclarator = TSESTree.VariableDeclarator & {
    id: TSESTree.ObjectPattern;
    init: TSESTree.Identifier;
};
type DisplayNameAssignmentExpression = TSESTree.AssignmentExpression & {
    type: "AssignmentExpression";
    left: TSESTree.MemberExpression & {
        property: TSESTree.Identifier & {
            name: "displayName";
        };
    };
    operator: "=";
    right: TSESTree.Literal;
};
declare const IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION = "ArrowFunctionExpression[body.type!='BlockStatement']";
declare const OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR: string;
declare const DISPLAY_NAME_ASSIGNMENT_EXPRESSION: string;

declare const SEL_DISPLAY_NAME_ASSIGNMENT_EXPRESSION: typeof DISPLAY_NAME_ASSIGNMENT_EXPRESSION;
type SEL_DisplayNameAssignmentExpression = DisplayNameAssignmentExpression;
declare const SEL_IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION: typeof IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION;
type SEL_ImplicitReturnArrowFunctionExpression = ImplicitReturnArrowFunctionExpression;
declare const SEL_OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR: typeof OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR;
type SEL_ObjectDestructuringVariableDeclarator = ObjectDestructuringVariableDeclarator;
declare namespace SEL {
  export { SEL_DISPLAY_NAME_ASSIGNMENT_EXPRESSION as DISPLAY_NAME_ASSIGNMENT_EXPRESSION, type SEL_DisplayNameAssignmentExpression as DisplayNameAssignmentExpression, SEL_IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION as IMPLICIT_RETURN_ARROW_FUNCTION_EXPRESSION, type SEL_ImplicitReturnArrowFunctionExpression as ImplicitReturnArrowFunctionExpression, SEL_OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR as OBJECT_DESTRUCTURING_VARIABLE_DECLARATOR, type SEL_ObjectDestructuringVariableDeclarator as ObjectDestructuringVariableDeclarator };
}

export { index$1 as JsxConfig, index as LanguagePreference, RE, Report, type RuleContext, type RuleDeclaration, type RuleFeature, type RulePreset, type RuleSeverity, SEL };
